#!bash
# the line above should be replaced with '# shellcheck shell=bash' once
# the used shellcheck is above 4.4
#
# This is a bash library, it has to be sourced not called.
#
# Authors:
#   - Martin Borer <martin.borer@tuwien.ac.at>
#
# gitlab-bot-tools-base-functions provides a number of base functions
# and a base setup of settings used by gitlab-bot-tools bash scripts.
#
# Internal global veriables are prefixed with GBTBF__,
# internal function with _gbtbf_

[ "$0" != "$BASH_SOURCE" ] || {
	echo "$0: this script must be sourced, not called" >&2
	exit 1
}

declare GBTBF__RUN_QUIET


######################################################################
# ensure that pip is in path for crontab
export PATH="$PATH:$HOME/.local/bin:$HOME/bin"


######################################################################
# check if python gitlab has the required version
function gbtbf_checkdep_pythongitlab() {
    bb_msg debug2 "check dependency python-gitlab"

    local currentver requiredver

    which gitlab > /dev/null || bb_quit 2 "'python-gitlab' seems not installed"

    currentver="$(gitlab --version)"
    requiredver="2.5.0"
    if [ "$(printf '%s\n' "$requiredver" "$currentver" | sort -V | head -n1)" = "$requiredver" ];
    then
        bb_msg debug2 "python-gitlab version: $currentver"
    else
        bb_quit 2 "'python-gitlab' has version $currentver," \
                "but at least version $requiredver required!"
    fi
    bb_msg debug3 "dependencie python-gitlab okay"
}


######################################################################
# check if python gitlab has the required version
function gbtbf_checkdep_jq() {
    bb_msg debug2 "check dependency jq"

    rpm --quiet -q jq || bb_quit 2 "'jq' seems not installed"
    bb_msg debug3 "dependencie jq okay"
}


######################################################################
# checks if in current run sudo scope may be accuired
function gbtbf_checkuserscope() {
    local gbtbf__response gbtbf__curuser

    ## get current user
    bb_msg debug3 "gitlab --output json current-user get"
    gbtbf__response=$(gitlab --output json current-user get) || {
        bb_msg error "could not communicate with gitlab"
        bb_quit 1
        }

    bb_msg debug3 "jq --raw-output '.username'"
    gbtbf__curuser=$(jq --raw-output '.username' \
                        <<<"$gbtbf__response")

    while [[ -n "${1:-}" ]]
    do
        case "$1" in
            "sudo" )
                bb_msg debug3 "gitlab current-user get" \
                       "--sudo \"$gbtbf__curuser\" > /dev/null"
                gitlab current-user get --sudo "$gbtbf__curuser" \
                       > /dev/null || {
                    bb_msg err "provided private_token has no sudo scope"
                    bb_quit 9
                }
                ;;
            *) return 90 ;;
        esac
        shift
    done
}


######################################################################
# setup logging / print stuff
function gbtbf_setuplogging() {
    (( $# == 4 )) || {
        bb_ierr "gbtbf_setuplogging:" \
                "called with $# instead of 4 arguments"
        return 2
    }

    local _logfile="$1"
    local -i _setuplogging_debug=$2
    local -i _setuplogging_quiet=$3
    local -i _setuplogging_verbose=$4

    local _logdir="${_logfile%/*}"
    local -i _logdircreated
    local -i _debuglevel

    # max verbosity is 3
    [[ $_setuplogging_verbose -le 3 ]] || \
        _setuplogging_verbose=3

    ## setup debug mode
    if [[ $_setuplogging_debug -eq 1 ]]
    then
        _debuglevel=3
    elif [[ $_setuplogging_verbose -gt 0 ]]
    then
        _debuglevel=$_setuplogging_verbose
    else
        _debuglevel=1
    fi

    ## setup logging with dir
    if [[ "$_logfile" != "" ]]
    then
        ## create logdir
        if [[ ! -d "$_logdir" ]]
        then
            mkdir -p "$_logdir" || \
                bb_quit 1 "could not create logdir '$_logdir'"
            _logdircreated=1
        fi

        # setup logging target
        bb_msg_set_target log file:"$_logfile" \
                          debug${_debuglevel}:emerg full notrace

        ## log logdir creation
        if [[ ${_logdircreated:-} -eq 1 ]]
        then
            bb_msg debug "logdir '$_logdir' created"
        else
            bb_msg debug3 "logdir '$_logdir' already existed"
        fi
    fi

    # log debug
    bb_msg debug2 "called with debug mode"

    ## setup verbosity
    if [[ $_setuplogging_verbose -gt 0 ]]
    then

        bb_msg_set_target verbose stderr \
                          debug${_setuplogging_verbose}:debug \
                          prog notrace
    fi
    ## log verbosity
    bb_msg debug2 "called with verbosity" \
           "$_setuplogging_verbose"


    ## setup quiet mode
    if [[ $_setuplogging_quiet -eq 1 ]]
    then
        # reset stdout/stderr targets
        bb_msg_del_target default_info_err
        bb_msg_set_target warning_err stderr warning:err prog notrace
    fi
    # log debug
    bb_msg debug2 "called with quiet mode"

    ## check if only -q or -v is given
    [[ $_setuplogging_quiet -gt 0 ]] && \
        [[ $_setuplogging_verbose -gt 0 ]] && \
        usage 1 "-q and -v are mutually exclusive, state only one"
    bb_msg debug3 "only -q or -v given"
}


######################################################################
## print msg to stdout if not explicitly quiet
function gbtbf_msg() {
    (( $# >= 0 )) || {
        bb_ierr "gbtbf_setuplogging:" \
                "called with 0 instead of at least 1 argument"
        return 2
    }
    [[ ${GBTBF__RUN_QUIET:-0} -eq 1 ]] ||
        printf "%s\n" "$*"
}


######################################################################
## get project path with namespace
function gbtbf_helper_getprojectpath() {
    (( $# == 2 )) || {
        bb_ierr "gbtbf_helper_getprojectpath:" \
                "called with $# instead of 2 arguments"
        return 2
    }

    local gbtbf__ret_vn="$1"
    local -i gbtbf__projectid=$2

    local gbtbf__name gbtbf__path_with_namespace gbtbf__response

    # get project information as json from gitlab
    bb_msg debug3 "gitlab --output json" \
           "--fields name,id,path_with_namespace" \
           "project get" \
           "--id $gbtbf__projectid"
    gbtbf__response=$(gitlab --output json \
                             --fields name,id,path_with_namespace \
                             project get \
                             --id "$gbtbf__projectid") || {
        bb_ierr "could not get the project information from gitlab"
        return 12
    }

    # get project with namespace
    bb_msg debug3 "jq --raw-output '.path_with_namespace' <<<\$gbtbf__response"
    gbtbf__path_with_namespace=$(jq --raw-output '.path_with_namespace' \
                                    <<<"$gbtbf__response")

    printf -v "$gbtbf__ret_vn" '%s' "$gbtbf__path_with_namespace"
}


######################################################################
## get project id
function gbtbf_helper_getprojectid() {
    (( $# == 2 )) || {
        bb_ierr "gbtbf_helper_getprojectid:" \
                "called with $# instead of 2 arguments"
        return 2
    }

    local gbtbf__ret_vn="$1"
    local gbtbf__path_with_namespace="${2,,}"

    local -i gbtbf__projectid
    local gbtbf__name gbtbf__response

    gbtbf__name="${gbtbf__path_with_namespace##*/}"

    # get project information as json from gitlab
    bb_msg debug3 "gitlab --output json" \
           "--fields name,id,path_with_namespace" \
           "project list" \
           "--all" \
           "--search $gbtbf__name"
    gbtbf__response=$(gitlab --output json \
                             --fields name,id,path_with_namespace \
                             project list \
                             --all \
                             --search "$gbtbf__name") || {
        bb_ierr "could not get the project information from gitlab"
        return 12
    }

    # get projectid
    bb_msg debug3 "jg '.[] |" \
           "select(.path_with_namespace==\"$gbtbf__path_with_namespace\") |" \
           ".id' <<<\$response"
    gbtbf__projectid=$(jq '.[] | select(.path_with_namespace=="'"$gbtbf__path_with_namespace"'") | .id' \
           <<<"$gbtbf__response")

    printf -v "$gbtbf__ret_vn" '%i' "$gbtbf__projectid"
}


######################################################################
## get issue title from iid
function gbtbf_helper_getissuetitle() {
    (( $# == 3 )) || {
        bb_ierr "gbtbf_helper_getissueiid_fromtitle:" \
                "called with $# instead of 3 arguments"
        return 2
    }

    local gbtbf__ret_vn="$1"
    local -i gbtbf__projectid=$2
    local -i gbtbf__issueiid=$3

    local gbtbf__issuetitle gbtbf__response

    # get project-issues from gitlab
    bb_msg debug3 "gitlab --output json --fields title,iid" \
           "project-issue get" \
           "--project-id $gbtbf__projectid" \
           "--iid $gbtbf__issueiid"
    gbtbf__response=$(gitlab --output json --fields title,iid \
                             project-issue get \
                             --project-id "$gbtbf__projectid" \
                             --iid "$gbtbf__issueiid") || {
        bb_ierr "could not get the project issue from gitlab," \
                "project-id: gbtbf__projectid, iid: $gbtbf__issueiid"
        return 12
    }

    # return issue title
    bb_msg debug3 "jq --raw-output '.title' <<<\$gbtbf__response"
    gbtbf__issuetitle=$(jq --raw-output '.title' <<<"$gbtbf__response")

    printf -v "$gbtbf__ret_vn" "%s" "$gbtbf__issuetitle"
}


######################################################################
## get issue iid from title with optional labels
function gbtbf_helper_getissueiids_fromtitle() {
    (( $# == 3 )) || (( $# == 4 )) || {
        bb_ierr "gbtbf_helper_getissueiid_fromtitle:" \
                "called with $# instead of 3 or 4 arguments"
        return 2
    }

    local gbtbf__ret_array_vn="$1"
    local -i gbtbf__projectid=$2
    local gbtbf__issuetitle="$3"
    local -a gbtbf__issuelabelsarray_vn
    (( $# == 3 )) || gbtbf__issuelabelsarray_vn="$4"

    local gbtbf__response
    local gbtbf__labels

    if (( $# == 3 ))
    then
        # get project-issues information as json from gitlab
        bb_msg debug3 "gitlab --output json --fields title,iid" \
               "project-issue list" \
               "-all" \
               "--project-id $gbtbf__projectid" \
               "--search \"$gbtbf__issuetitle\"" \
               "--sort asc"
        gbtbf__response=$(gitlab --output json --fields title,iid \
                                 project-issue list \
                                 --all \
                                 --project-id "$gbtbf__projectid" \
                                 --search "$gbtbf__issuetitle" \
                                 --sort asc) || {
            bb_ierr "could not get the project-issue list from gitlab"
            return 12
        }
    elif (( $# == 4 ))
    then
        bb_array_join_by gbtbf__labels "$gbtbf__issuelabelsarray_vn" ,

        # get project-issues information as json from gitlab
        bb_msg debug3 "gitlab --output json --fields title,iid" \
               "project-issue list" \
               "--all" \
               "--project-id $gbtbf__projectid" \
               "--search \"$gbtbf__issuetitle\"" \
               "--labels \"$gbtbf__labels\"" \
               "--sort asc"
        gbtbf__response=$(gitlab --output json --fields title,iid \
                                 project-issue list \
                                 --all \
                                 --project-id "$gbtbf__projectid" \
                                 --search "$gbtbf__issuetitle" \
                                 --labels "$gbtbf__labels" \
                                 --sort asc) || {
            bb_ierr "could not get the project-issue list from gitlab"
            return 12
        }
    fi

    bb_var_want_array "$gbtbf__ret_array_vn"

    # get issueiids
    bb_msg debug3 "jq '.[] |" \
           "select(.title==\"$gbtbf__issuetitle\") | .iid' <<<\$response"
    readarray -t "$gbtbf__ret_array_vn" \
              < <(jq '.[] | select(.title=="'"$gbtbf__issuetitle"'") | .iid' \
                     <<<"$gbtbf__response")
}


######################################################################
## get user id
function gbtbf_helper_getuserid_from_username() {
    (( $# == 2 )) || {
        bb_ierr "gbtbf_helper_getuserid_from_username:" \
                "called with $# instead of 2 arguments"
        return 2
    }

    local gbtbf__ret_vn="$1"
    local gbtbf__username="$2"

    local -i gbtbf__userid
    local gbtbf__response

    # get user information as json from gitlab
    bb_msg debug3 "gitlab --output json user list" \
           "--username $gbtbf__username"
    gbtbf__response=$(gitlab --output json user list \
                             --username "$gbtbf__username") \
        || bb_quit 11 "could not get the user information from gitlab"

    # return userid
    bb_msg debug3 "jq --raw-output '.[] | .id' <<<\$gbtbf__response"
    gbtbf__userid=$(jq --raw-output '.[] | .id' <<<"$gbtbf__response")

    printf -v "$gbtbf__ret_vn" "%s" "$gbtbf__userid"
}
